@@ -33,6 +33,7 @@
 #include "Cumulant.h"
 #include "CASPT2.h"
 #include "MPIchemps2.h"
+#include "EdmistonRuedenberg.h"
 
 using std::string;
 using std::ifstream;
@@ -206,6 +207,21 @@ double CheMPS2::CASSCF::caspt2( const int Nelectrons, const int TwoS, const int
    HamAS->getVmat()->broadcast( MPI_CHEMPS2_MASTER );
    #endif
 
+   // Reorder the orbitals based on the Fiedler vector of the exchange matrix
+   if ( scf_options->getWhichActiveSpace() == 3 ){
+      int * dmrg2ham = new int[ nOrbDMRG ];
+      if ( am_i_master ){
+         EdmistonRuedenberg * theLocalizer = new EdmistonRuedenberg( HamAS->getVmat(), iHandler->getGroupNumber() );
+         theLocalizer->FiedlerGlobal( dmrg2ham );
+         delete theLocalizer;
+      }
+      #ifdef CHEMPS2_MPI_COMPILATION
+      MPIchemps2::broadcast_array_int( dmrg2ham, nOrbDMRG, MPI_CHEMPS2_MASTER );
+      #endif
+      Prob->setup_reorder_custom( dmrg2ham );
+      delete [] dmrg2ham;
+   }
+
    double E_CASSCF = 0.0;
    double * three_dm = new double[ tot_dmrg_power6 ];
    double * contract = new double[ tot_dmrg_power6 ];
