@@ -23,6 +23,7 @@
 #include <iostream>
 #include <string>
 #include <fstream>
+#include <sys/stat.h>
 
 #include "Irreps.h"
 #include "TwoIndex.h"
@@ -31,7 +32,6 @@
 #include "MyHDF5.h"
 
 using std::cout;
-using std::cerr;
 using std::endl;
 using std::string;
 using std::ifstream;
@@ -69,15 +69,15 @@ CheMPS2::Hamiltonian::Hamiltonian( const string filename, const int psi4groupnum
 
 }
 
-CheMPS2::Hamiltonian::Hamiltonian(const bool fileh5, const string main_file, const string file_tmat, const string file_vmat){
+CheMPS2::Hamiltonian::Hamiltonian( const bool fileh5, const string main_file, const string file_tmat, const string file_vmat ){
 
-   if (fileh5){
+   if ( fileh5 ){
       CreateAndFillFromH5( main_file, file_tmat, file_vmat );
    } else {
-      cerr << "CheMPS2::Hamiltonian::Hamiltonian( false, const string , const string , const string ) was deprecated." << endl;
+      cout << "CheMPS2::Hamiltonian::Hamiltonian( false, const string , const string , const string ) was deprecated." << endl;
       assert( fileh5 == true );
    }
-   
+
 }
 
 CheMPS2::Hamiltonian::~Hamiltonian(){
@@ -294,6 +294,15 @@ void CheMPS2::Hamiltonian::CreateAndFillFromH5(const string file_parent, const s
 
 void CheMPS2::Hamiltonian::CreateAndFillFromFCIDUMP( const string fcidumpfile ){
 
+    struct stat file_info;
+    const bool on_disk = (( fcidumpfile.length() > 0 ) && ( stat( fcidumpfile.c_str(), &file_info ) == 0 ));
+    if ( on_disk == false ){
+       cout << "CheMPS2::Hamiltonian : Unable to find FCIDUMP file " << fcidumpfile << "!" << endl;
+    }
+    assert( on_disk );
+
+    cout << "CheMPS2::Hamiltonian : Reading FCIDUMP file " << fcidumpfile << endl;
+
     const int nIrreps = SymmInfo.getNumberOfIrreps();
     int * psi2molpro = new int[ nIrreps ];
     SymmInfo.symm_psi2molpro( psi2molpro );
@@ -301,7 +310,7 @@ void CheMPS2::Hamiltonian::CreateAndFillFromFCIDUMP( const string fcidumpfile ){
     ifstream thefcidump( fcidumpfile.c_str() );
     string line, part;
     int pos, pos2;
-    
+
     // Get the number of orbitals
     getline( thefcidump, line ); // &FCI NORB= X,NELEC= Y,MS2= Z,   
     pos  = line.find( "NORB" );
@@ -350,24 +359,9 @@ void CheMPS2::Hamiltonian::CreateAndFillFromFCIDUMP( const string fcidumpfile ){
         orb2indexSy[cnt] = irrep2num_orb[orb2irrep[cnt]];
         irrep2num_orb[orb2irrep[cnt]]++;
     }
-    Tmat = new TwoIndex(SymmInfo.getGroupNumber(),irrep2num_orb);
-    Vmat = new FourIndex(SymmInfo.getGroupNumber(),irrep2num_orb);
+    Tmat = new TwoIndex(  SymmInfo.getGroupNumber(), irrep2num_orb ); // Constructor ends with Clear(); call
+    Vmat = new FourIndex( SymmInfo.getGroupNumber(), irrep2num_orb ); // Constructor ends with Clear(); call
 
-    // Clear the Hamiltonian
-    Econst = 0.0;
-    for ( int orb1 = 0; orb1 < L; orb1++ ){
-        for ( int orb2 = orb1; orb2 < L; orb2++ ){
-            const int irrepprod12 = SymmInfo.directProd( orb2irrep[ orb1 ] , orb2irrep[ orb2 ] );
-            if ( irrepprod12 == 0 ){ setTmat( orb1, orb2, 0.0 ); }
-            for ( int orb3 = orb1; orb3 < L; orb3++ ){
-                for ( int orb4 = orb2; orb4 < L; orb4++ ){
-                    const int irrepprod34 = SymmInfo.directProd( orb2irrep[ orb3 ] , orb2irrep[ orb4 ] );
-                    if ( irrepprod12 == irrepprod34 ){ setVmat( orb1, orb2, orb3, orb4, 0.0 ); }
-                }
-            }
-        }
-    }
-    
     // Read the Hamiltonian in
     bool stop = false;
     while ( stop == false ){
@@ -423,6 +417,8 @@ void CheMPS2::Hamiltonian::CreateAndFillFromFCIDUMP( const string fcidumpfile ){
     delete [] psi2molpro;
     thefcidump.close();
 
+    cout << "CheMPS2::Hamiltonian : Finished reading FCIDUMP file " << fcidumpfile << endl;
+
 }
 
 void CheMPS2::Hamiltonian::writeFCIDUMP( const string fcidumpfile, const int Nelec, const int TwoS, const int TargetIrrep ) const{
