@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,10 +23,6 @@
 
 #include "Initialize.h"
 
-CheMPS2::Initialize::Initialize(){ }
-
-CheMPS2::Initialize::~Initialize(){ }
-
 void CheMPS2::Initialize::Init(){
 
    std::cout.precision(15);
